@@ -10,5 +10,6 @@ from controlSBML.simulate_system import simulateSystem, makeStateVector
 from controlSBML.timeseries import Timeseries, TimeseriesSer
 mat2TS = Timeseries.mat2TS
 BIOMODELS_DCT = {
-      823: "https://www.ebi.ac.uk/biomodels/model/download/BIOMD0000000823.2?filename=Varusai2018.xml"
+      823: "https://www.ebi.ac.uk/biomodels/model/download/BIOMD0000000823.2?filename=Varusai2018.xml",
+      206: "https://www.ebi.ac.uk/biomodels/model/download/BIOMD0000000206.2?filename=BIOMD0000000206_url.xml",
 }
