@@ -7,7 +7,7 @@ Outputs are chemical species.
 
 Notes:
 1. Reaction enzymes are identified by the SBML reaction ID.
-2. The Jacobian is only recalculated if there is a change in timepoint
+2. The Jacobian is only recalculated if there is a change in time
 """
 
 
@@ -41,6 +41,7 @@ class ControlBase(object):
         model_reference: str
             string, SBML file or Roadrunner object
         input_names: name (id) of reaction whose flux is being controlled
+                     or the name of a chemical species
         output_names: list-str
             output species
         is_reduced: bool
@@ -50,11 +51,11 @@ class ControlBase(object):
         # Iinitial model calculations
         self.model_reference = model_reference
         self.roadrunner = makeRoadrunner(self.model_reference)
-        self._jacobian_timepoint = TIMEPOINT_NULL
-        self._jacobian_df = None
-        # Set defaults
         self.species_names = list(
               self.roadrunner.getFullStoichiometryMatrix().rownames)
+        self._jacobian_time = TIMEPOINT_NULL
+        self._jacobian_df = None
+        # Set defaults.
         self.depeendent_names = list(
               set(self.species_names).symmetric_difference(self.state_names))
         if not set(self.state_names) <= set(self.species_names):
@@ -68,7 +69,7 @@ class ControlBase(object):
             self.input_names = []
         else:
             self.input_names = input_names
-        self.input_names = self._sortList(self.reaction_names, self.input_names)
+        #self.input_names = self._sortList(self.reaction_names, self.input_names)
         self.num_input = len(self.input_names)
         if output_names is None:
             output_names = self.species_names
@@ -81,7 +82,6 @@ class ControlBase(object):
         self.roadrunner.reset()
         # Validation checks
         if not set(self.state_names) <= set(self.species_names):
-            import pdb; pdb.set_trace()
             text = "State does not include some spaces.\n"
             text += "  Species are: %s" % str(self.species_names)
             text += "  States are: %s" % str(self.state_names)
@@ -91,6 +91,12 @@ class ControlBase(object):
             text = "Outputs must be species. The following outputs are not species"
             text += "The following outputs are not species: %s" % str(diff)
             raise ValueError(text)
+        possible_input_names = set(self.state_names).union(self.reaction_names)
+        if not set(self.input_names) <= set(possible_input_names):
+            diff = list(set(self.input_names).difference(possible_input_names))
+            text = "Inputs must be a species or a reaction."
+            text += "   Invalid names are: %s" % str(diff)
+            raise ValueError(text)
 
     @property
     def B_df(self):
@@ -195,7 +201,7 @@ class ControlBase(object):
         -------
         pd.DataFrame, species_names
         """
-        if np.isclose(self.getTime(), self._jacobian_timepoint):
+        if np.isclose(self.getTime(), self._jacobian_time):
             if self._jacobian_df is not None:
                 return self._jacobian_df
         if self.is_reduced:
@@ -209,7 +215,7 @@ class ControlBase(object):
             raise RuntimeError("Jacobian is not square!")
         names = list(jacobian_mat.colnames)
         self._jacobian_df = pd.DataFrame(jacobian_mat, columns=names, index=names)
-        self._jacobian_timepoint = self.getTime()
+        self._jacobian_time = self.getTime()
         return self._jacobian_df
 
     @property
@@ -231,7 +237,7 @@ class ControlBase(object):
 
     def setTime(self, time):
         self.roadrunner.reset()
-        self._jacobian_timepoint = TIMEPOINT_NULL
+        self._jacobian_time = TIMEPOINT_NULL
         if time > 0.01:
             _ = self.roadrunner.simulate(0.0, time)
 
@@ -354,21 +360,38 @@ class ControlBase(object):
         new_super_lst = list(super_lst)
         return sorted(sub_lst, key=lambda v: new_super_lst.index(v))
 
+    def separateSpeciesReactionInputs(self):
+        species_inputs = [n for n in self.input_names if n in self.species_names]
+        reaction_inputs = [n for n in self.input_names if n in self.reaction_names]
+        return species_inputs, reaction_inputs
+
     def _makeBDF(self):
         """
         Constructs a dataframe for the B matrix.
+        The columns must be in the same order as the input_names.
 
         Returns
         -------
         np.ndarray (n X p), where p = len(input_names)
         """
         if len(self.input_names) > 0:
+            # Determine which inputs are reactions and which inputs are species
+            species_inputs, reaction_inputs = self.separateSpeciesReactionInputs()
+            # Construct the matrix for species inputs
+            if len(species_inputs) > 0:
+                eye = np.eye(self.num_state)
+                B_species_df = pd.DataFrame(eye, columns=self.state_names,
+                      index=self.state_names)
+                B_species_df = B_species_df[species_inputs]
+                B_df = B_species_df
+            if len(reaction_inputs) > 0:
+                B_reaction_df = self.full_stoichiometry_df[reaction_inputs]
+                B_df = B_reaction_df
             # Select the columns needed from the stoichiometry matrix
-            B_df = self.full_stoichiometry_df[self.input_names]
-            # Subset to the states
-            sub_names = list(set(B_df.index).intersection(self.state_names))
-            sub_names = self._sortList(self.state_names, sub_names)
-            B_df = B_df.loc[sub_names, :]
+            # Merge the two
+            if (len(species_inputs) > 0) and (len(reaction_inputs) > 0):
+                B_df = pd.concat([B_species_df, B_reaction_df], axis=1)
+            B_df = B_df[self.input_names]
         else:
             ncol = 1
             B_mat = np.repeat(0, self.num_state)
@@ -377,7 +400,7 @@ class ControlBase(object):
         #
         return B_df
 
-    def makeStateSpace(self, A_mat=None, B_mat=None,
+    def makeStateSpace(self, time=None, A_mat=None, B_mat=None,
           C_mat=None, D_mat=None):
         """
         Creates a state space control object for
@@ -387,6 +410,7 @@ class ControlBase(object):
         ----------
         The default values of the matrices are calculated in the constructor.
         These can be overridden.
+        time: float (time at which Jacobian is obtained)
         A_mat: np.array(n X n) or DataFrame
         B_mat: np.array(n X p) or DataFrame
         C_mat: np.array(q X n) or DataFrame
@@ -408,7 +432,12 @@ class ControlBase(object):
         C_mat = df2Mat(C_mat)
         D_mat = df2Mat(D_mat)
         if A_mat is None:
+            if time is not None:
+                current_time = self.getTime()
+                self.setTime(time)
             A_mat = self.jacobian_df.values
+            if time is not None:
+                self.setTime(current_time)
         if B_mat is None:
             B_df = self.B_df
             if B_df is None:
@@ -426,11 +455,8 @@ class ControlBase(object):
                 ncol = np.shape(B_mat)[1]
             D_mat = np.repeat(0, nrow*ncol)
             D_mat = np.reshape(D_mat, (nrow, ncol))
-        try:
-            ss = control.StateSpace(A_mat, B_mat, C_mat, D_mat)
-        except:
-            import pdb; pdb.set_trace()
-        return control.StateSpace(A_mat, B_mat, C_mat, D_mat)
+        ss = control.StateSpace(A_mat, B_mat, C_mat, D_mat)
+        return ss
 
     def makeNonlinearIOSystem(self, name, effector_dct=None):
         """
@@ -450,18 +476,60 @@ class ControlBase(object):
         """
         return ctl.NonlinearIOSystem(name, self, effector_dct=effector_dct)
 
-    def makeTransferFunction(self):
+    @staticmethod
+    def reduceTransferFunction(tf):
+        """
+        Reduces the order of a transfer function if trailing zeroes.
+
+        Parameters
+        ----------
+        tf: control.TransferFunction
+        
+        Returns
+        -------
+        tf: control.TransferFunction
+        """
+        def findOrderOfFirstNonzeroDigit(polynomial):
+            for idx in range(len(polynomial)):
+                pos = len(polynomial) - idx - 1
+                if not np.isclose(polynomial[pos], 0):
+                    return idx
+            return len(polynomial) - 1
+        def reduceOrder(polynomial, new_order):
+            pos = len(polynomial) - new_order
+            return polynomial[0:pos]
+        #
+        numerator = tf.num[0][0]
+        denominator = tf.den[0][0]
+        lowest_order = min(findOrderOfFirstNonzeroDigit(numerator),
+              findOrderOfFirstNonzeroDigit(denominator))
+        #
+        new_numerator = reduceOrder(numerator, lowest_order)
+        new_denominator = reduceOrder(denominator, lowest_order)
+        new_tf = control.TransferFunction(new_numerator, new_denominator)
+        return new_tf
+
+    def makeTransferFunction(self, time=None):
         """
         Creates a transfer function for the system. Verifies that there
-        is a single input and a single output.
+        is a single input and a single output. Reduces the order of the
+        transfer function as needed.
+        
+        Parameters
+        ----------
+        time: float (time at which Jacobian is obtained)
         
         Returns
         -------
         control.TransferFunction
         """
+        # Validity checks
         if len(self.input_names) != 1:
             raise ValueError("Must have exactly one input.")
         if len(self.output_names) != 1:
             raise ValueError("Must have exactly one output.")
-        state_space = self.makeStateSpace()
-        return control.ss2tf(state_space)
+        # Get initial transfer function
+        state_space = self.makeStateSpace(time=time)
+        tf = control.ss2tf(state_space)
+        #
+        return self.reduceTransferFunction(tf)
